
// \file Camera.cpp
// --------------------------------------------------------------------------
// (C) Copyright, 2006
// Computer Graphics Group, University of Siegen, Germany
// All Rights Reserved.
//
// This is unpublished proprietary source code.
// The contents of this file may not be disclosed to third parties,
// copied or duplicated in any form, in whole or in part, without the
// prior written permission of the copyright holder.
// --------------------------------------------------------------------------
 
#include "Camera.h"

#define _USE_MATH_DEFINES 1

#include <float.h>
#include <math.h>
#include <QtOpenGL/qgl.h>
 #include <GL/glu.h>
//#include <OpenGL/glu.h> //MAC_OS_X

#define DEG2RAD(a) (((a)/180.0)*M_PI)


/**
 * Constructor.
 *
 * @param polar     Initial polar angle
 * @param azimut    Initial azimut angle
 * @param distance  Initial distance to the coordinate center
 */
Camera::Camera(double polar, double azimut, double distance) :
m_polar(polar), m_azimut(azimut), m_distance(distance)
{
  m_lookAt[0] = 0.0;
  m_lookAt[1] = 0.0;
  m_lookAt[2] = 0.0;    
}


/**
 * Standard destructor.
 */
Camera::~Camera(void)
{
}


/** Update position **/
void Camera::update(void) 
{
  m_position[0] = m_lookAt[0] + m_distance*cos(DEG2RAD(m_polar))*sin(DEG2RAD(m_azimut));
  m_position[1] = m_lookAt[1] + m_distance*sin(DEG2RAD(m_polar));
  m_position[2] = m_lookAt[2] + m_distance*cos(DEG2RAD(m_polar))*cos(DEG2RAD(m_azimut));
}


/**
 * OpenGL camera setup during rendering pass.
 */
void Camera::setup(void)
{
  gluLookAt(
    m_position[0], // eyepoint 
    m_position[1], 
    m_position[2],
    
    m_lookAt[0], // center 
    m_lookAt[1], 
    m_lookAt[2],
    
    0.0, 1.0, 0.0  // up vector
  );
}


/**
 * Set pitch angle.
 */
void Camera::setPolar(double polar) 
{ 
  if (polar >= 90.0)
    m_polar = 89.99;
  else if (polar <= -90.0)
    m_polar = -89.99;
  else 
    m_polar = polar; 

  update();
};


/**
 * Set rotation angle
 */
void Camera::setAzimut(double azimut) 
{  
  m_azimut = azimut - 360.0*floor(azimut/360.0);

  if (m_azimut < 0.0)
    m_azimut += 360.0; 

  update();
};


/**
 * Set camera distance.
 */
void Camera::setDistance(double distance) { 
  if (distance < FLT_EPSILON) 
    distance = FLT_EPSILON;

  m_distance = distance; 

  update();
};

