
// \file GLView.qt.cpp
// --------------------------------------------------------------------------
// \version $LastChangedRevision$
// \date    $LastChangedDate$
// \author  $LastChangedBy$
// --------------------------------------------------------------------------
// Project: Graphik-Praktikum - Rotations-Geometrie
// --------------------------------------------------------------------------
// (C) Copyright, 2006
// Computer Graphics Group, University of Siegen, Germany
// All Rights Reserved.
//
// This is unpublished proprietary source code.
// The contents of this file may not be disclosed to third parties,
// copied or duplicated in any form, in whole or in part, without the
// prior written permission of the copyright holder.
// --------------------------------------------------------------------------


#include "GLView.qt.h"




char     g_mouseButton = 0;
int      g_oldX = 0, g_oldY = 0;

// defines
#define LBUTTON 1
#define MBUTTON 2
#define RBUTTON 4

#define MAX_ENVIRONMENT 50.0
#define MAX_ANIMATIONSPEED 4

// Minimum and maximum radius of planets
#define RADIUS_MIN_THRESHOLD 0.15
#define RADIUS_MAX_THRESHOLD 0.25

// Minimum and maximum distance between planets
#define DISTANCE_MIN_THRESHOLD 0.6
#define DISTANCE_MAX_THRESHOLD 1.5

// Minimum and maximum rotation speed of planets
#define ROTATION_MIN_THRESHOLD 2.0
#define ROTATION_MAX_THRESHOLD 20.0


const float GLView::s_bgrColor[3] = {0.0f, 0.0f, 0.5f};

// Gouraud values
const float g_l[]   = {0.0f, 40.0f, 0.0f, 1.0f};
const float g_l_a[] = {0.5f, 0.5f, 0.5f, 1.0f};
const float g_l_d[] = {1.0f, 1.0f, 0.0f, 1.0f};
const float g_l_s[] = {1.0f, 1.0f, 1.0f, 1.0f};
const float g_0[] = {0.7f, 0.7f, 0.7f, 1.0f};

/**
 * Constructor
 */
GLView::GLView(QWidget *parent) 
 : QGLWidget(parent)
{
  // create objects
  m_camera = new Camera(0.0, 0.0, 6.0);
  m_camera->setLookAt(1, 0);
}

/**
 * Destructor 
 */
GLView::~GLView()
{
  delete m_camera;
}

/**
 * Handles new input events when a mouse button is pressed.
 *
 * @param m   mouse event object
 */
void GLView::mousePressEvent(QMouseEvent *m) 
{
  m_lastX = m->x();
  m_lastY = m->y();
}


/**
 * Handles new input events when the mouse is moved.
 *
 * @param m   mouse event object
 */
void GLView::mouseMoveEvent(QMouseEvent *m)
{
  // get delta movement to last position
  int deltaX = m->x() - m_lastX;
  int deltaY = m->y() - m_lastY;

  // check if there has been a movement at all
  if ((deltaX == 0) && (deltaY == 0))
    return;

  // get the buttons involved
  MouseButtons state = m->buttons();
    
  bool lb = (state & LeftButton)  == LeftButton;
  bool mb = (state & MidButton)   == MidButton;
  bool rb = (state & RightButton) == RightButton;

  // process action
  if (lb) {
    m_camera->setPolar(m_camera->polar() + deltaY);
    m_camera->setAzimut(m_camera->azimut() - deltaX);
  }
  else if (rb) {
    m_camera->setDistance(m_camera->distance() - 0.1*deltaY);
  }
  
  // update state
  m_lastX = m->x();
  m_lastY = m->y();

  updateGL();
}


/**
 * Initialize OpenGL context.
 */ 
void GLView::initializeGL()
{
#ifdef _DEBUG
  glClearColor(0.2, 0.2, 0.2, 1.0);
#else
  glClearColor(0.0, 0.0, 0.0, 1.0);
#endif

  glShadeModel(GL_SMOOTH);

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glEnable(GL_BLEND);

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}


/**
 * Handle window resize event.
 *
 * @param width    new window width
 * @param height   new window height
 */ 
void GLView::resizeGL(int width, int height)
{
  // update GL state
  glViewport(0, 0, (GLsizei) width, (GLsizei) height);

  // init OpenGL projection  here
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  ///////////////////////
  // Same as -> gluPerspective(45,(double) width/height, 1.5, 2000.0);
  float aspect =  (float) width/height;
  float fovY = 45;
  float zNear = 0.01f;
  float zFar =  1.5*MAX_ENVIRONMENT;

  GLdouble fW, fH;
  fH = tan( fovY / 360 * M_PI ) * zNear;
  fW = fH * aspect;
  glFrustum( -fW, fW, -fH, fH, zNear, zFar );
  //
  ///////////////////////

  glMatrixMode(GL_MODELVIEW);
}


/**
 * OpenGL draw routine.
 */ 
void GLView::paintGL()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  m_camera->setup();

  // draw frame
  glBegin(GL_LINES);
    glColor3f(1.f,0,0);
    glVertex3f(0,0,0);
    glVertex3f(1.f,0,0);

    glColor3f(0,1.f,0);
    glVertex3f(0,0,0);
    glVertex3f(0,1.f,0);

    glColor3f(0,0,1.f);
    glVertex3f(0,0,0);
    glVertex3f(0,0,1.f);
  glEnd();
}
