
// \file GLView.qt.h
// --------------------------------------------------------------------------
// \version $LastChangedRevision$
// \date    $LastChangedDate$
// \author  $LastChangedBy$
// --------------------------------------------------------------------------
// Project: Graphik-Praktikum - Rotations-Geometrie
// --------------------------------------------------------------------------
// (C) Copyright, 2006
// Computer Graphics Group, University of Siegen, Germany
// All Rights Reserved.
//
// This is unpublished proprietary source code.
// The contents of this file may not be disclosed to third parties,
// copied or duplicated in any form, in whole or in part, without the
// prior written permission of the copyright holder.
// --------------------------------------------------------------------------


#ifndef GLVIEW_H
#define GLVIEW_H


#include <assert.h>
#include "Camera.h"
#include <vector>
#include <iostream>

#include <time.h>
#include <QtOpenGL/QtOpenGL>
#include <QtGui/QtGui>

using namespace Qt;

/**
 * OpenGL widget used for polyline definition.
 *
 * Control points can be either created or deleted by pressing the right 
 * mouse button. By pressing the left button existing control points can
 * be selected and modified.
 */
class GLView : public QGLWidget
{
	// place this macro here to include the standard QT object defitions
  Q_OBJECT

public:  

	//---------------------------------------------------------------------------
	//! \name                      Instantiation
	//! \{
	//---------------------------------------------------------------------------    

	//! Constructor
	GLView(QWidget *parent=0);

	//! Destructor
	virtual ~GLView();

 protected:
    
  //---------------------------------------------------------------------------
  //! \name                        Input Handler
  //! \{
  //---------------------------------------------------------------------------    

	//! Handle mouse button event.
	void mousePressEvent(QMouseEvent *m);

	//! Handle mouse move event.
	void mouseMoveEvent(QMouseEvent *m);

  //!\}------------------------------------------------------------------------


  //---------------------------------------------------------------------------
  //! \name                           OpenGL 
  //! \{
  //--------------------------------------------------------------------------- 

	//! Initialize OpenGL context.
	void initializeGL();

  //! Handle window resize event.
	void resizeGL( int, int );

  //! OpenGL drap routine.
	void paintGL();
	

  void setupScene(void);




  //!\}------------------------------------------------------------------------

  int                m_lastX;					//!< last mouse coordinate
  int                m_lastY;					//!< last mouse coordinate

  float			         m_lightAngle;			//!< position of light
  int                s_nrTrees;   			//!< number of trees
  int				         *m_randPhi;	            //!< Phi pos
  int				         *m_randTheta;	        //!< Theta pos
  static const int   s_range = 100;					//!< 

  Camera            *m_camera;

  static const float s_bgrColor[3];    //!< background color

};

#endif // GLVIEW_H
